@@ -412,7 +412,8 @@ uand:				    write_unaligned(write_and, write_and_masked);
 #undef next_x_chunk
 	return 0;
 	/* Handle the funny cases that aren't supposed to happen. */
-funny:	return (invert ? -1 : mem_mono_fill_rectangle(dev, x, y, w, h, zero));
+funny:	return (invert ? gs_note_error(-1) :
+		mem_mono_fill_rectangle(dev, x, y, w, h, zero));
 #undef optr
 #endif				/* !USE_COPY_ROP */
 }
