@@ -1,4 +1,4 @@
-/* Copyright (C) 1994, 1995, 1996 Aladdin Enterprises.  All rights reserved.
+/* Copyright (C) 1994, 1995, 1996, 1997 Aladdin Enterprises.  All rights reserved.
   
   This file is part of Aladdin Ghostscript.
   
@@ -214,7 +214,8 @@ bits_replicate_vertically(byte *data, uint height, uint raster,
 {	byte *dest = data;
 	uint h = replicated_height;
 	uint size = raster * height;
-	while ( h >= height )
+
+	while ( h > height )
 	  {	memcpy(dest + size, dest, size);
 		dest += size;
 		h -= height;
