@@ -1,4 +1,4 @@
-/* Copyright (C) 1995 Aladdin Enterprises.  All rights reserved.
+/* Copyright (C) 1995, 1997 Aladdin Enterprises.  All rights reserved.
   
   This file is part of Aladdin Ghostscript.
   
@@ -29,6 +29,7 @@ typedef enum {
 	gs_cap_square = 2,
 	gs_cap_triangle = 3	/* not supported by PostScript */
 } gs_line_cap;
+#define gs_line_cap_max 3
 
 /* Line join values */
 typedef enum {
@@ -38,5 +39,6 @@ typedef enum {
 	gs_join_none = 3,	/* not supported by PostScript */
 	gs_join_triangle = 4	/* not supported by PostScript */
 } gs_line_join;
+#define gs_line_join_max 4
 
 #endif					/* gslparam_INCLUDED */
