@@ -1,4 +1,4 @@
-/* Copyright (C) 1989, 1996 Aladdin Enterprises.  All rights reserved.
+/* Copyright (C) 1989, 1996, 1997 Aladdin Enterprises.  All rights reserved.
   
   This file is part of Aladdin Ghostscript.
   
@@ -22,6 +22,7 @@
 #ifndef gxdevice_INCLUDED
 #  define gxdevice_INCLUDED
 
+#include "stdio_.h"		/* for FILE */
 #include "gsdcolor.h"
 #include "gsmatrix.h"
 #include "gsiparam.h"		/* requires gsmatrix.h */
@@ -139,6 +140,22 @@ dev_page_proc_install(gx_default_install);
 dev_page_proc_begin_page(gx_default_begin_page);
 dev_page_proc_end_page(gx_default_end_page);
 
+/* Define default pages sizes. */
+/* U.S. letter paper (8.5" x 11"). */
+#define DEFAULT_WIDTH_10THS_US_LETTER 85
+#define DEFAULT_HEIGHT_10THS_US_LETTER 110
+/* A4 paper (210mm x 297mm).  The dimensions are off by a few mm.... */
+#define DEFAULT_WIDTH_10THS_A4 83
+#define DEFAULT_HEIGHT_10THS_A4 117
+/* Choose a default.  A4 may be set in the makefile. */
+#ifdef A4
+#  define DEFAULT_WIDTH_10THS DEFAULT_WIDTH_10THS_A4
+#  define DEFAULT_HEIGHT_10THS DEFAULT_HEIGHT_10THS_A4
+#else
+#  define DEFAULT_WIDTH_10THS DEFAULT_WIDTH_10THS_US_LETTER
+#  define DEFAULT_HEIGHT_10THS DEFAULT_HEIGHT_10THS_US_LETTER
+#endif
+
 /* ---------------- Device structure ---------------- */
 
 /*
@@ -249,9 +266,15 @@ dev_page_proc_end_page(gx_default_end_page);
 	sizeof(devtype), ptr_procs, dev_name,\
 	0 /*memory*/, stype, open_init() /*stype, is_open, cached*/
 /* color_info goes here */
+/*
+ * The MetroWerks compiler has some bizarre bug that produces a spurious
+ * error message if the width and/or height are defined as 0 below,
+ * unless we use the +/- workaround in the next macro.
+ */
 #define std_device_part2_(width, height, x_dpi, y_dpi)\
 	width, height,\
-	{ (width) * 72.0 / (x_dpi), (height) * 72.0 / (y_dpi) },\
+	{ (((width) * 72.0 + 0.5) - 0.5) / (x_dpi),\
+	  (((height) * 72.0 + 0.5) - 0.5) / (y_dpi) },\
 	{ 0, 0, 0, 0 }, 0/*false*/, { x_dpi, y_dpi }, { x_dpi, y_dpi }
 /* offsets and margins go here */
 #define std_device_part3_()\
@@ -272,7 +295,7 @@ dev_page_proc_end_page(gx_default_end_page);
  *	  6 for margins/offset).
  *	
  */ 
-#define std_device_body_with_macros_(dtype, pprocs, dname, w, h, xdpi, ydpi, stype, open_init, dci_macro, margins_macro)\
+#define std_device_body_with_macros_(dtype, pprocs, dname, stype, w, h, xdpi, ydpi, open_init, dci_macro, margins_macro)\
 	std_device_part1_(dtype, pprocs, dname, stype, open_init),\
 	dci_macro(),\
 	std_device_part2_(w, h, xdpi, ydpi),\
@@ -280,14 +303,14 @@ dev_page_proc_end_page(gx_default_end_page);
 	std_device_part3_()
 
 #define std_device_std_body(dtype, pprocs, dname, w, h, xdpi, ydpi)\
-	std_device_body_with_macros_(dtype, pprocs, dname,\
+	std_device_body_with_macros_(dtype, pprocs, dname, 0,\
 	  w, h, xdpi, ydpi,\
-	  0, open_init_closed, dci_black_and_white_, no_margins_)
+	  open_init_closed, dci_black_and_white_, no_margins_)
 
 #define std_device_std_body_open(dtype, pprocs, dname, w, h, xdpi, ydpi)\
-	std_device_body_with_macros_(dtype, pprocs, dname,\
+	std_device_body_with_macros_(dtype, pprocs, dname, 0,\
 	  w, h, xdpi, ydpi,\
-	  0, open_init_open, dci_black_and_white_, no_margins_)
+	  open_init_open, dci_black_and_white_, no_margins_)
 
 #define std_device_full_body(dtype, pprocs, dname, w, h, xdpi, ydpi, ncomp, depth, mg, mc, dg, dc, xoff, yoff, lm, bm, rm, tm)\
 	std_device_part1_(dtype, pprocs, dname, 0, open_init_closed),\
@@ -296,11 +319,16 @@ dev_page_proc_end_page(gx_default_end_page);
 	offset_margin_values(xoff, yoff, lm, bm, rm, tm),\
 	std_device_part3_()
 
+#define std_device_dci_type_body(dtype, pprocs, dname, stype, w, h, xdpi, ydpi, ncomp, depth, mg, mc, dg, dc)\
+	std_device_part1_(dtype, pprocs, dname, stype, open_init_closed),\
+	dci_values(ncomp, depth, mg, mc, dg, dc),\
+	std_device_part2_(w, h, xdpi, ydpi),\
+	offset_margin_values(0, 0, 0, 0, 0, 0),\
+	std_device_part3_()
+
 #define std_device_dci_body(dtype, pprocs, dname, w, h, xdpi, ydpi, ncomp, depth, mg, mc, dg, dc)\
-	std_device_full_body(dtype, pprocs, dname,\
-	  w, h, xdpi, ydpi,\
-	  ncomp, depth, mg, mc, dg, dc,\
-	  0, 0, 0, 0, 0, 0)
+	std_device_dci_type_body(dtype, pprocs, dname, 0,\
+	  w, h, xdpi, ydpi, ncomp, depth, mg, mc, dg, dc)
 
 #define std_device_color_full_body(dtype, pprocs, dname, w, h, xdpi, ydpi, depth, max_value, dither, xoff, yoff, lm, bm, rm, tm)\
 	std_device_part1_(dtype, pprocs, dname, 0, open_init_closed),\
@@ -573,22 +601,22 @@ typedef struct gs_param_list_s gs_param_list;
 #define dev_proc_draw_thin_line(proc)\
   dev_t_proc_draw_thin_line(proc, gx_device)
 
-		/* Added in release 3.66 (as stubs), */
-		/* changed in 3.68 */
+		/* Added in release 3.66 (as stubs); */
+		/* changed in 3.68; */
+		/* begin_image and image_data changed in 4.30 */
 
 #define dev_t_proc_begin_image(proc, dev_t)\
   int proc(P9(dev_t *dev,\
     const gs_imager_state *pis, const gs_image_t *pim,\
-    gs_image_format_t format, gs_image_shape_t shape,\
+    gs_image_format_t format, const gs_int_rect *prect,\
     const gx_drawing_color *pdcolor, const gx_clip_path *pcpath,\
     gs_memory_t *memory, void **pinfo))
 #define dev_proc_begin_image(proc)\
   dev_t_proc_begin_image(proc, gx_device)
 
 #define dev_t_proc_image_data(proc, dev_t)\
-  int proc(P8(dev_t *dev,\
-    void *info, const byte **planes, uint raster,\
-    int x, int y, int width, int height))
+  int proc(P6(dev_t *dev,\
+    void *info, const byte **planes, int data_x, uint raster, int height))
 #define dev_proc_image_data(proc)\
   dev_t_proc_image_data(proc, gx_device)
 
@@ -618,6 +646,13 @@ typedef struct gs_param_list_s gs_param_list;
 #define dev_proc_strip_copy_rop(proc)\
   dev_t_proc_strip_copy_rop(proc, gx_device)
 
+		/* Added in release 4.20 */
+
+#define dev_t_proc_get_clipping_box(proc, dev_t)\
+  void proc(P2(dev_t *dev, gs_fixed_rect *pbox))
+#define dev_proc_get_clipping_box(proc)\
+  dev_t_proc_get_clipping_box(proc, gx_device)
+
 /* Define the device procedure vector template proper. */
 
 #define gx_device_proc_struct(dev_t)\
@@ -657,6 +692,7 @@ typedef struct gs_param_list_s gs_param_list;
 	dev_t_proc_end_image((*end_image), dev_t);\
 	dev_t_proc_strip_tile_rectangle((*strip_tile_rectangle), dev_t);\
 	dev_t_proc_strip_copy_rop((*strip_copy_rop), dev_t);\
+	dev_t_proc_get_clipping_box((*get_clipping_box), dev_t);\
 }
 
 /* A generic device procedure record. */
@@ -701,9 +737,11 @@ struct gx_device_s {
 };
 extern_st(st_device);
 struct_proc_finalize(gx_device_finalize);	/* public for subclasses */
+/* We use vacuous enum/reloc procedures, rather than 0, so that */
+/* gx_device can have subclasses. */
 #define public_st_device()	/* in gsdevice.c */\
   gs_public_st_complex_only(st_device, gx_device, "gx_device",\
-    0, 0, 0, gx_device_finalize)
+    0, gs_no_struct_enum_ptrs, gs_no_struct_reloc_ptrs, gx_device_finalize)
 #define st_device_max_ptrs 0
 
 /* Enumerate or relocate a pointer to a device. */
@@ -785,11 +823,13 @@ void gx_device_set_media_size(P3(gx_device *dev, floatp media_width, floatp medi
  */
 
 /* Macros for fill_rectangle and [strip_]tile_rectangle. */
-#define fit_fill_xyw(dev, x, y, w, h)\
+#define fit_fill_xy(dev, x, y, w, h)\
 	if ( (x | y) < 0 )\
 	{	if ( x < 0 ) w += x, x = 0;\
 		if ( y < 0 ) h += y, y = 0;\
-	}\
+	}
+#define fit_fill_xyw(dev, x, y, w, h)\
+	fit_fill_xy(dev, x, y, w, h);\
 	if ( x > dev->width - w ) w = dev->width - x
 #define fit_fill_y(dev, y, h)\
 	if ( y < 0 ) h += y, y = 0
@@ -801,9 +841,11 @@ void gx_device_set_media_size(P3(gx_device *dev, floatp media_width, floatp medi
 #define fit_fill_xywh(dev, x, y, w, h)\
 	fit_fill_xyw(dev, x, y, w, h);\
 	fit_fill_h(dev, y, h)
-#define fit_fill(dev, x, y, w, h)\
+#define if_fit_fill_empty(dev, x, y, w, h)\
 	fit_fill_xywh(dev, x, y, w, h);\
-	if ( w <= 0 || h <= 0 ) return 0
+	if ( w <= 0 || h <= 0 )
+#define fit_fill(dev, x, y, w, h)\
+	if_fit_fill_empty(dev, x, y, w, h) return 0
 
 /* Macro for copy_mono and copy_color. */
 #define fit_copy_xw(dev, data, data_x, raster, id, x, y, w, h)\
@@ -866,6 +908,8 @@ dev_proc_strip_tile_rectangle(gx_default_strip_tile_rectangle);
 dev_proc_strip_copy_rop(gx_no_strip_copy_rop);		/* gives error */
 extern dev_proc_strip_copy_rop((*gx_default_strip_copy_rop_proc));
 dev_proc_strip_copy_rop(gx_default_strip_copy_rop);	/* calls ...proc */
+dev_proc_get_clipping_box(gx_default_get_clipping_box);
+dev_proc_get_clipping_box(gx_get_largest_clipping_box);
 
 /* Color mapping routines for black-on-white, gray scale, true RGB, */
 /* and true CMYK color. */
@@ -907,6 +951,7 @@ dev_proc_image_data(gx_forward_image_data);
 dev_proc_end_image(gx_forward_end_image);
 dev_proc_strip_tile_rectangle(gx_forward_strip_tile_rectangle);
 extern dev_proc_strip_copy_rop((*gx_forward_strip_copy_rop_proc));
+dev_proc_get_clipping_box(gx_forward_get_clipping_box);
 
 /* Convert the device procedures to the proper form (see above). */
 void	gx_device_set_procs(P1(gx_device *));
@@ -923,6 +968,11 @@ void	gx_device_forward_color_procs(P1(gx_device_forward *));
 void gx_device_no_output(P1(gs_state *));
 void gx_set_device_only(P2(gs_state *, gx_device *));
 
+/* Open the output file for a device. */
+int gx_device_open_output_file(P5(const gx_device *dev, const char *fname,
+				  bool binary, bool positionable,
+				  FILE **pfile));
+
 /* Close a device. */
 int gs_closedevice(P1(gx_device *));
 
