@@ -1,4 +1,4 @@
-/* Copyright (C) 1989, 1996 Aladdin Enterprises.  All rights reserved.
+/* Copyright (C) 1989, 1996, 1997 Aladdin Enterprises.  All rights reserved.
   
   This file is part of Aladdin Ghostscript.
   
@@ -59,10 +59,13 @@ gs_get_real_stdio(FILE *stdfiles[3])
 /* Define the default instance of the interpreter. */
 /* Currently, this is the *only possible* instance, because most of */
 /* the places that need to take an explicit instance argument don't. */
-private gs_main_instance the_gs_main_instance =
-  { gs_main_instance_default_init_values };
-gs_main_instance *gs_main_instance_default()
-{	return &the_gs_main_instance;
+private gs_main_instance the_gs_main_instance;
+gs_main_instance *
+gs_main_instance_default(void)
+{	/* Determine whether the instance has been initialized. */
+	if ( the_gs_main_instance.memory_chunk_size == 0 )
+	  the_gs_main_instance = gs_main_instance_init_values;
+	return &the_gs_main_instance;
 }
 
 /* The only reason we export gs_exit_status is so that window systems */
@@ -82,6 +85,10 @@ extern ref gs_emulator_name_array[];
 /* ------ Forward references ------ */
 
 private int gs_run_init_file(P3(gs_main_instance *, int *, ref *));
+#ifdef DEBUG
+private void print_usage(P3(const gs_main_instance *, gs_dual_memory_t *,
+			    const char *));
+#endif
 
 /* ------ Initialization ------ */
 
@@ -102,6 +109,9 @@ gs_main_init0(gs_main_instance *minst, FILE *in, FILE *out, FILE *err,
 	/* because it detects attempts to run 80N86 executables (N>0) */
 	/* on incompatible processors. */
 	gp_init();
+#ifdef DEBUG
+	gp_get_usertime(minst->base_time);
+#endif
 	/* Initialize the imager. */
 	gs_lib_init0(gs_stdout);
 	/* Initialize the file search paths. */
@@ -178,6 +188,10 @@ gs_main_init2(gs_main_instance *minst)
 		}
 		minst->init_done = 2;
 	   }
+#ifdef DEBUG
+	if ( gs_debug_c(':') )
+	  print_usage(minst, &gs_imemory, "Start");
+#endif
 }
 
 /* ------ Search paths ------ */
@@ -365,7 +379,7 @@ gs_main_run_string_with_length(gs_main_instance *minst, const char *str,
 int
 gs_main_run_string_begin(gs_main_instance *minst, int user_errors,
   int *pexit_code, ref *perror_object)
-{	const char *setup = "{.needinput}bind 0().subfiledecode cvx exec";
+{	const char *setup = ".runstringbegin";
 	ref rstr;
 	int code;
 
@@ -530,6 +544,10 @@ gs_main_finit(gs_main_instance *minst, int exit_status, int code)
 	 */
 	gs_exit_status = exit_status;	/* see above */
 
+#ifdef DEBUG
+	if ( gs_debug_c(':') )
+	  print_usage(minst, &gs_imemory, "Final");
+#endif
 	/* Do the equivalent of a restore "past the bottom". */
 	/* This will release all memory, close all open files, etc. */
 	if ( minst->init_done >= 1 )
@@ -548,6 +566,36 @@ gs_exit(int exit_status)
 
 /* ------ Debugging ------ */
 
+#ifdef DEBUG
+/* Print usage statistics. */
+private void
+print_usage(const gs_main_instance *minst, gs_dual_memory_t *dmem,
+  const char *msg)
+{	ulong allocated = 0, used = 0;
+	long utime[2];
+
+	gp_get_usertime(utime);
+	{ int i;
+
+	  for ( i = 0; i < countof(dmem->spaces.indexed); ++i ) {
+	    gs_ref_memory_t *mem = dmem->spaces.indexed[i];
+
+	    if ( mem != 0 && (i == 0 || mem != dmem->spaces.indexed[i - 1]) ) {
+	      gs_memory_status_t status;
+
+	      gs_memory_status((gs_memory_t *)mem, &status);
+	      allocated += status.allocated;
+	      used += status.used;
+	    }
+	  }
+	}
+	dprintf4("%% %s time = %g, memory allocated = %lu, used = %lu\n",
+		 msg, utime[0] - minst->base_time[0] +
+		   (utime[1] - minst->base_time[1]) / 1000000000.0,
+		 allocated, used);
+}
+#endif
+
 /* Dump the stacks after interpretation */
 void
 gs_debug_dump_stack(int code, ref *perror_object)
