@@ -1,4 +1,4 @@
-/* Copyright (C) 1989, 1995 Aladdin Enterprises.  All rights reserved.
+/* Copyright (C) 1989, 1995, 1997 Aladdin Enterprises.  All rights reserved.
   
   This file is part of Aladdin Ghostscript.
   
@@ -22,6 +22,7 @@
 #include "string_.h"
 #include "ghost.h"
 #include "gsstruct.h"
+#include "gxobj.h"		/* for o_set_unmarked */
 #include "errors.h"
 #include "inamedef.h"
 #include "imemory.h"			/* for isave.h */
@@ -71,9 +72,11 @@ private const far_data byte nt_1char_names[128] = {
 #undef q8
 };
 
-gs_private_st_simple(st_name_sub_table, name_sub_table, "name_sub_table");
-gs_private_st_composite(st_name_table, name_table,
-  "name_table", name_table_enum_ptrs, name_table_reloc_ptrs);
+/* Structure descriptors */
+gs_private_st_composite(st_name_sub_table, name_sub_table, "name_sub_table",
+  name_sub_enum_ptrs, name_sub_reloc_ptrs);
+gs_private_st_composite(st_name_table, name_table, "name_table",
+  name_table_enum_ptrs, name_table_reloc_ptrs);
 
 /* The one and only name table (for now). */
 private name_table *the_nt;
@@ -138,7 +141,7 @@ name_init(ulong count, gs_memory_t *mem)
 	}
 	/* Reconstruct the free list. */
 	nt->free = 0;
-	name_gc_cleanup(NULL);
+	name_trace_finish(NULL);
 	/* Register the name table root. */
 	gs_register_struct_root(mem, &the_nt_root, (void **)&the_nt,
 				"name table");
@@ -365,14 +368,17 @@ name_index_ptr_sub_table(uint index, name *pname)
 {	return pname - (index & nt_sub_index_mask);
 }
 
-/* Clean up the name table after a garbage collection, by removing */
-/* names that aren't marked, and relocating name string pointers */
-/* (the latter only if gcst != 0). */
+/*
+ * Clean up the name table after the trace/mark phase of a garbage
+ * collection, by removing names that aren't marked.  gcst == NULL indicates
+ * we're doing this for initialization or restore rather than for a GC.
+ */
 void
-name_gc_cleanup(gc_state_t *gcst)
+name_trace_finish(gc_state_t *gcst)
 {	name_table *nt = the_nt;
 	uint *phash = &nt->hash[0];
-	register uint i;
+	uint i;
+
 	for ( i = 0; i < nt_hash_size; phash++, i++ )
 	   {	uint prev = 0;
 		name *pnprev;
@@ -381,14 +387,7 @@ name_gc_cleanup(gc_state_t *gcst)
 		{	name *pname = name_index_ptr_inline(nt, nidx);
 			uint next = name_next_index(nidx, pname);
 			if ( pname->mark )
-			{	if ( !pname->foreign_string && gcst != NULL)
-				  {	gs_const_string nstr;
-					nstr.data = pname->string_bytes;
-					nstr.size = pname->string_size;
-					gs_reloc_const_string(&nstr, gcst);
-					pname->string_bytes = nstr.data;
-				  }
-				prev = nidx;
+			{	prev = nidx;
 				pnprev = pname;
 			}
 			else
@@ -407,8 +406,16 @@ name_gc_cleanup(gc_state_t *gcst)
 	}
 	/* Reconstruct the free list. */
 	nt->free = 0;
-	for ( i = nt->sub_count - 1; ; --i )
-	  { name_scan_sub(nt, i, true);
+	for ( i = nt->sub_count; i--; )
+	  { name_sub_table *sub = nt->sub_tables[i];
+
+	    if ( sub != 0 )
+	      { name_scan_sub(nt, i, true);
+	        if ( nt->sub_tables[i] == 0 && gcst != 0 )
+		  { /* Mark the just-freed sub-table as unmarked. */
+		    o_set_unmarked((obj_header_t *)sub - 1);
+		  }
+	      }
 	    if ( i == 0 )
 	      break;
 	  }
@@ -427,7 +434,7 @@ void
 name_restore(alloc_save_t *save)
 {	name_table *nt = the_nt;
 	/* We simply mark all names older than the save, */
-	/* and let name_gc_cleanup sort everything out. */
+	/* and let name_trace_finish sort everything out. */
 	uint si;
 	for ( si = 0; si < nt->sub_count; ++si )
 	  if ( nt->sub_tables[si] != 0 )
@@ -444,7 +451,7 @@ name_restore(alloc_save_t *save)
 		      !alloc_is_since_save(pname->string_bytes, save);
 		}
 	    }
-	name_gc_cleanup(NULL);
+	name_trace_finish(NULL);
 }
 
 /* ------ Internal procedures ------ */
@@ -539,11 +546,16 @@ name_scan_sub(name_table *nt, uint sub_index, bool free_empty)
 	  {	/* No marked entries, free the sub-table. */
 		gs_free_object(nt->memory, sub, "name_scan_sub");
 		nt->sub_tables[sub_index] = 0;
-		if ( sub_index == nt->sub_count - 1 )
-		  {	nt->sub_count = sub_index;
-			if ( nt->sub_next == sub_index )
-			  nt->sub_next--;
-		  }
+		if ( sub_index == nt->sub_count - 1 ) {
+		  /* Back up over a final run of deleted sub-tables. */
+		  do {
+		    --sub_index;
+		  } while ( nt->sub_tables[sub_index] == 0 );
+		  nt->sub_count = sub_index + 1;
+		  if ( nt->sub_next > sub_index )
+		    nt->sub_next = sub_index;
+		} else if ( nt->sub_next == sub_index )
+		  nt->sub_next--;
 	  }
 }
 
@@ -568,3 +580,23 @@ private RELOC_PTRS_BEGIN(name_table_reloc_ptrs) {
 	 * permanent dictionaries, at the very end of garbage collection.
 	 */
 } RELOC_PTRS_END
+#undef ntptr
+
+private ENUM_PTRS_BEGIN_PROC(name_sub_enum_ptrs) {
+	return 0;
+} ENUM_PTRS_END_PROC
+private RELOC_PTRS_BEGIN(name_sub_reloc_ptrs) {
+	name *pname = ((name_sub_table *)vptr)->names;
+	uint i;
+
+	for ( i = 0; i < nt_sub_size; ++pname, ++i ) {
+	  if ( pname->string_bytes != 0 && !pname->foreign_string ) {
+	    gs_const_string nstr;
+
+	    nstr.data = pname->string_bytes;
+	    nstr.size = pname->string_size;
+	    gs_reloc_const_string(&nstr, gcst);
+	    pname->string_bytes = nstr.data;
+	  }
+	}
+} RELOC_PTRS_END
