@@ -1,4 +1,4 @@
-/* Copyright (C) 1989, 1996 Aladdin Enterprises.  All rights reserved.
+/* Copyright (C) 1989, 1996, 1997 Aladdin Enterprises.  All rights reserved.
   
   This file is part of Aladdin Ghostscript.
   
@@ -38,13 +38,11 @@ public_st_stream_state();	/* default */
 private ENUM_PTRS_BEGIN(stream_enum_ptrs) return 0;
 	case 0:
 		if ( st->foreign )
-		  *pep = NULL;
+		  ENUM_RETURN(NULL);
 		else if ( st->cbuf_string.data != 0 )
-		  { ENUM_RETURN_STRING_PTR(stream, cbuf_string);
-		  }
+		  ENUM_RETURN_STRING_PTR(stream, cbuf_string);
 		else
-		  *pep = st->cbuf;
-		break;
+		  ENUM_RETURN(st->cbuf);
 	ENUM_PTR(1, stream, strm);
 	ENUM_PTR(2, stream, prev);
 	ENUM_PTR(3, stream, next);
@@ -224,8 +222,11 @@ s_disable(register stream *s)
 	s->end_status = EOFC;
 	s->modes = 0;
 	s->cbuf_string.data = 0;
-	s->cursor.r.ptr = s->cursor.r.limit = (const byte *)0 - 1;
-	s->cursor.w.limit = (byte *)0 - 1;
+	/* The pointers in the next two statements should really be */
+	/* initialized to ([const] byte *)0 - 1, but some very picky */
+	/* compilers complain about this. */
+	s->cursor.r.ptr = s->cursor.r.limit = 0;
+	s->cursor.w.limit = 0;
 	s->procs.close = s_std_null;
 	/* Clear pointers for GC */
 	s->strm = 0;
@@ -601,12 +602,19 @@ private int
 swritebuf(stream *s, stream_cursor_read *pbuf, bool last)
 {	stream *prev = 0;
 	stream *curr = s;
-	int depth = 1;		/* depth of nesting in non-temp streams */
-	int level = 0;		/* depth of recursion */
-	int top_level = 0;	/* level below which all streams have */
-				/* returned 0 when called with last = true */
+	int depth = 0;		/* depth of nesting in non-temp streams */
 	int status;
 
+	/*
+	 * The handling of EOFC is a little tricky.  There are two
+	 * invariants that keep it straight:
+	 *	- We only pass last = true to a stream if either it is
+	 * the first stream in the pipeline, or it is a temporary stream
+	 * below the first stream and the stream immediately above it has
+	 * end_status = EOFC.
+	 *	- We never unwind the recursion past a stream with
+	 * end_status < 0.
+	 */
 	for ( ; ; )
 	{	for ( ; ; )
 		{	/* Descend into the recursion. */
@@ -619,7 +627,9 @@ swritebuf(stream *s, stream_cursor_read *pbuf, bool last)
 			 * the top-level stream and any temporary streams
 			 * immediately below it.
 			 */
-			bool end = last && depth <= 1 && level == top_level;
+			bool end = last &&
+			  (prev == 0 ||
+			   (depth <= 1 && prev->end_status == EOFC));
 
 			if ( strm == 0 )
 			  cw.ptr = 0, cw.limit = 0, pw = &cw;
@@ -632,38 +642,45 @@ swritebuf(stream *s, stream_cursor_read *pbuf, bool last)
 			if_debug4('s', "[s]write process 0x%lx, nr=%u, nw=%u, end=%d\n",
 				  (ulong)curr, (uint)(pr->limit - pr->ptr),
 				  (uint)(pw->limit - pw->ptr), end);
-			status = (*curr->procs.process)(curr->state, pr, pw,
-							end);
-			if_debug4('s', "[s]after write 0x%lx, nr=%u, nw=%u, status=%d\n",
+			status = curr->end_status;
+			if ( status < 0 )
+			  break;
+			status = (*curr->procs.process)(curr->state,
+							pr, pw, end);
+			if_debug5('s', "[s]after write 0x%lx, nr=%u, nw=%u, end=%d, status=%d\n",
 				  (ulong)curr, (uint)(pr->limit - pr->ptr),
-				  (uint)(pw->limit - pw->ptr), status);
-			if ( strm == 0 || status < 0 )
+				  (uint)(pw->limit - pw->ptr), end, status);
+			if ( status == 0 && end )
+			  status = EOFC;
+			if ( status == EOFC || status == ERRC )
+			  curr->end_status = status;
+			if ( strm == 0 || (status < 0 && status != EOFC) )
 			  break;
 			if ( status == 1 )
 			  end = false;
 			else
 			{	/* Keep going if we are closing */
 				/* a filter with a sub-stream. */
-				/* We know status == 0. */
+				/* We know status == 0 or EOFC. */
 				if ( !end || !strm->is_temp )
 				  break;
-				/* This level is finished, don't come back. */
-				top_level = level + 1;
 			}
 			status = strm->end_status;
 			if ( status < 0 )
 			  break;
 			move_ahead(curr, prev);
 			stream_compact(curr, false);
-			++level;
 			if ( !curr->is_temp )
 			  ++depth;
 		}
 		/* Unwind from the recursion. */
 		curr->end_status = (status >= 0 ? 0 : status);
-		if ( level <= top_level )
-		  { /* All streams above here were called with last = true */
-		    /* and returned 0: finish unwinding and then return. */
+		if ( status < 0 || prev == 0 )
+		  { /*
+		     * All streams above here were called with last = true
+		     * and returned 0 or EOFC: finish unwinding and then
+		     * return.
+		     */
 		    while ( prev )
 		      { move_back(curr, prev);
 			curr->end_status = (status >= 0 ? 0 : status);
@@ -671,7 +688,6 @@ swritebuf(stream *s, stream_cursor_read *pbuf, bool last)
 		    return status;
 		  }
 		move_back(curr, prev);
-		--level;
 		if ( !curr->is_temp )
 		  --depth;
 	}
