@@ -1,4 +1,4 @@
-/* Copyright (C) 1995 Aladdin Enterprises.  All rights reserved.
+/* Copyright (C) 1995, 1997 Aladdin Enterprises.  All rights reserved.
   
   This file is part of Aladdin Ghostscript.
   
@@ -24,9 +24,26 @@
 #include "gsstruct.h"
 #include "strimpl.h"
 #include "szlibx.h"
+#include "zconf.h"
 
 public_st_zlib_state();
 
+#define ss ((stream_zlib_state *)st)
+
+/* Set defaults for stream parameters. */
+void
+s_zlib_set_defaults(stream_state *st)
+{	ss->windowBits = MAX_WBITS;
+	ss->no_wrapper = false;
+	ss->level = Z_DEFAULT_COMPRESSION;
+	ss->method = Z_DEFLATED;
+	/* DEF_MEM_LEVEL should be in zlib.h or zconf.h, but it isn't. */
+	ss->memLevel = min(MAX_MEM_LEVEL, 8);
+	ss->strategy = Z_DEFAULT_STRATEGY;
+}
+
+#undef ss
+
 /* Provide zlib-compatible allocation and freeing functions. */
 void *
 s_zlib_alloc(void *mem, uint items, uint size)
