@@ -1,4 +1,4 @@
-/* Copyright (C) 1992, 1995, 1996 Aladdin Enterprises.  All rights reserved.
+/* Copyright (C) 1992, 1995, 1996, 1997 Aladdin Enterprises.  All rights reserved.
   
   This file is part of Aladdin Ghostscript.
   
@@ -386,7 +386,7 @@ zsetcieabcspace(register os_ptr op)
 			  "setcolorspace(CIEBasedABC)");
 	code = cie_abc_param(op, pcie, &procs.cie);
 	if ( code < 0 )
-	{	rc_free_struct(pcie, mem, "setcolorspace(CIEBasedABC)");
+	{	rc_free_struct(pcie, "setcolorspace(CIEBasedABC)");
 		return code;
 	}
 	cs.params.abc = pcie;
@@ -415,7 +415,7 @@ zsetcieaspace(register os_ptr op)
 	     (code = dict_float_array_param(op, "MatrixA", 3, (float *)&pcie->MatrixA, (const float *)&MatrixA_default)) != 3 ||
 	     (code = cie_lmnp_param(op, &pcie->common, &procs.cie)) < 0
 	   )
-	{	rc_free_struct(pcie, mem, "setcolorspace(CIEBasedA)");
+	{	rc_free_struct(pcie, "setcolorspace(CIEBasedA)");
 		return (code < 0 ? code : gs_note_error(e_rangecheck));
 	}
 	pcie->DecodeA = DecodeA_default;
@@ -570,7 +570,7 @@ cie_cache_finish(os_ptr op)
 	/* See above for the container + offset representation of */
 	/* the pointer to the cache. */
 	pcache = (cie_cache_floats *)(r_ptr(esp - 1, char) + esp->value.intval);
-	code = num_params(op, gx_cie_cache_size, &pcache->values[0]);
+	code = float_params(op, gx_cie_cache_size, &pcache->values[0]);
 	if_debug3('c', "[c]cache 0x%lx base=%g, factor=%g:\n",
 		  (ulong)pcache, pcache->params.base, pcache->params.factor);
 	if ( code < 0 )
@@ -578,7 +578,7 @@ cie_cache_finish(os_ptr op)
 		/* Handle the parameters one-by-one. */
 		uint i;
 		for ( i = 0; i < gx_cie_cache_size; i++ )
-		  {	code = real_param(ref_stack_index(&o_stack,
+		  {	code = float_param(ref_stack_index(&o_stack,
 						gx_cie_cache_size - 1 - i),
 					  &pcache->values[i]);
 			if ( code < 0 )
