@@ -1,4 +1,4 @@
-/* Copyright (C) 1989, 1995, 1996 Aladdin Enterprises.  All rights reserved.
+/* Copyright (C) 1989, 1995, 1996, 1997 Aladdin Enterprises.  All rights reserved.
   
   This file is part of Aladdin Ghostscript.
   
@@ -498,7 +498,10 @@ zflushfile(register os_ptr op)
 	{	pop(1);
 		return 0;
 	}
-	return handle_write_status(status, op, NULL, zflushfile);
+	return
+	  (s_is_writing(s) ?
+	   handle_write_status(status, op, NULL, zflushfile) :
+	   handle_read_status(status, op, NULL, zflushfile));
 }
 
 /* <file> resetfile - */
@@ -670,6 +673,12 @@ BEGIN_OP_DEFS(zfileio_op_defs) {
 	{"3.writecvp", zwritecvp},
 	{"2writehexstring", zwritehexstring},
 	{"2writestring", zwritestring},
+		/* Internal operators */
+	{"3%zreadhexstring_continue", zreadhexstring_continue},
+	{"3%zwritehexstring_continue", zwritehexstring_continue},
+	{"3%zreadstring_continue", zreadstring_continue},
+	{"3%zreadline_continue", zreadline_continue},
+	{"4%zwritecvp_continue", zwritecvp_continue},
 END_OP_DEFS(0) }
 
 /* ------ Non-operator routines ------ */
